﻿/*
    Forza Studio: Forza Motorsport resource extraction tool.

    Copyright (C) 2012 Mike Davis (http://codeescape.com/)

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be included 
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ForzaStudio
{
    public class ForzaCarBin : ForzaResource
    {
        #region Fields
        public EndianStream Stream;
        public ForzaCarSection[] Sections;
        public uint TypeId;
        #endregion

        #region Constructors / Destructor
        public ForzaCarBin(string archivePath, string filePath)
            : base(filePath, archivePath)
        {
            Initialize();
        }

        public ForzaCarBin(string filePath)
            : base(filePath)
        {
            Initialize();
        }

        ~ForzaCarBin()
        {
            Dispose();
        }
        #endregion

        #region Methods
        public void Initialize()
        {
            Stream = new EndianStream(GetDataStream(), EndianType.BigEndian);
            switch (Version = GetVersion(Stream))
            {
                case ForzaVersion.Three:
                    {
                        TypeId = Stream.ReadUInt32();
                        if (Stream.ReadUInt32() != 0xD)
                        {
                            if (TypeId == 1)
                            {
                                Stream.Position += 0x100;
                            }
                            else if (TypeId == 2)
                            {
                                Stream.Position += 0x14;
                            }
                        }
                        Stream.Position += 8;
                        Stream.Seek(Stream.ReadUInt32() * 0x8C, SeekOrigin.Current);
                        Stream.Position += 0x1AC;
                        uint unknownCount2 = Stream.ReadUInt32() * 2;
                        for (int i = 0; i < unknownCount2; i++)
                        {
                            Stream.Seek(Stream.ReadUInt32() * 4, SeekOrigin.Current);
                        }
                        Stream.Position += 4;

                        // parse through each section
                        Sections = new ForzaCarSection[Stream.ReadUInt32()];
                        for (int i = 0; i < Sections.Length; i++)
                        {
                            Sections[i] = new ForzaCarSection(this);
                        }

                        break;
                    }
                case ForzaVersion.Four:
                    {
                        TypeId = Stream.ReadUInt32();
                        if (TypeId == 1)
                        {
                            Stream.Position += 0x35C;
                            Stream.Seek(Stream.ReadUInt32() * 4, SeekOrigin.Current);
                            Stream.Position += 8;
                        }
                        else if (TypeId == 2)
                        {
                            Stream.Position += 0x15C;
                            uint unk1 = Stream.ReadUInt32();
                            Stream.Position += 4;
                            Stream.Position += unk1 * 0x8C;
                            Stream.Position += 0x340;
                            uint unkCount = Stream.ReadUInt32() * 2;
                            for (int i = 0; i < unkCount; i++)
                            {
                                Stream.Seek(Stream.ReadUInt32() * 4, SeekOrigin.Current);
                            }
                            Stream.Position += 4;
                        }
                        else if (TypeId == 3)
                        {
                            Stream.Position += 0x398;
                            uint unkCount = Stream.ReadUInt32() * 2;
                            for (int i = 0; i < unkCount; i++)
                            {
                                Stream.Seek(Stream.ReadUInt32() * 4, SeekOrigin.Current);
                            }
                            Stream.Position += 4;
                        }
                        uint partCount = Stream.ReadUInt32();
                        List<ForzaCarSection> partsList = new List<ForzaCarSection>();
                        for (int i = 0; i < partCount; i++)
                        {
                            try
                            {
                                partsList.Add(new ForzaCarSection(this));
                            }
                            catch (Exception ex)
                            {
                            
                            }
                        }

                        // add them to the final parts list
                        Sections = new ForzaCarSection[partsList.Count];
                        for (int i = 0; i < Sections.Length; i++)
                        {
                            Sections[i] = partsList[i];
                        }

                        if (Sections.Length == 0)
                            throw new Exception("analyze this!");

                        string name = this.Name;



                        break;
                    }
                default: throw new NotSupportedException();
            }
            Stream.Dispose();
        }

        /// <summary>
        /// Provides an extremely quick and hacky way to tell carbin versions apart from one another.
        /// </summary>
        /// <param name="stream">The stream to the carbin object.</param>
        /// <returns>The game version.</returns>
        private ForzaVersion GetVersion(Stream stream)
        {
            // preserve old stream position
            long oldPosition = stream.Position;

            // get version
            ForzaVersion version = ForzaVersion.Unknown;
            if (Stream.Length > 512)  // should be much larger than 512 bytes...
            {
                Stream.Position = 0;

                // take some magic samples
                uint firstDword = Stream.ReadUInt32();
                uint secondDword = Stream.ReadUInt32();
                stream.Seek(0x70, SeekOrigin.Begin);
                uint f3test = Stream.ReadUInt32();
                stream.Seek(0x104, SeekOrigin.Begin);
                uint f2test = Stream.ReadUInt32();
                stream.Seek(0x154, SeekOrigin.Begin);
                uint f4test = Stream.ReadUInt32();

                // check for versions in exactly this order
                if (firstDword == 1 && secondDword == 0x2ca || firstDword == 2 && f2test == 0x2ca)
                {
                    version = ForzaVersion.Two;
                }
                else if (firstDword == 1 && secondDword == 0x10 || firstDword == 2 && f4test == 0x10 || firstDword == 3)
                {
                    version = ForzaVersion.Four;
                }
                else if (firstDword == 2 && (secondDword == 0 || secondDword == 1) || firstDword == 1 && f3test == 0)
                {
                    version = ForzaVersion.Three;
                }
            }
            
            // restore old stream position and return the version
            stream.Position = oldPosition;
            return version;
        }

        /// <summary>
        /// Disposes of the object.
        /// </summary>
        public override void Dispose()
        {
            if (Stream != null) Stream.Dispose();
            Sections = null;
        }
        #endregion
    }
}
