﻿/*
    Forza Studio: Forza Motorsport resource extraction tool.

    Copyright (C) 2012 Mike Davis (http://codeescape.com/)

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be included 
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Collections;

namespace ForzaStudio
{
    public class ForzaCarSubSection
    {
        #region Fields
        public string Name;
        public EndianStream Stream;
        public ForzaCarSection Parent;
        public ForzaVertex[] Vertices;
        public int[] Indices;
        public int Lod;
        public IndexType IndexType;
        public int FaceCount;
        public int VertexCount;
        public float XUVOffset = 0;
        public float XUVScale = 1;
        public float YUVOffset = 0;
        public float YUVScale = 1;
        public float XUV2Offset = 0;
        public float XUV2Scale = 1;
        public float YUV2Offset = 0;
        public float YUV2Scale = 1;
        #endregion

        #region Constructor
        public ForzaCarSubSection(ForzaCarSection parentSection)
        {
            Parent = parentSection;
            Stream = parentSection.Stream;
            switch (parentSection.Parent.Version)
            {
                case ForzaVersion.Three:
                    {
                        Utilities.AssertEquals(Stream.ReadUInt32(), 2);
                        uint unkType = Stream.ReadByte();   // 0, 1
                        Utilities.AssertEquals(Stream.ReadUInt32(), 2);
                        break;
                    }
                case ForzaVersion.Four:
                    {
                        Stream.Position += 5;
                        XUVOffset = Stream.ReadSingle();
                        XUVScale = Stream.ReadSingle();
                        YUVOffset = Stream.ReadSingle();
                        YUVScale = Stream.ReadSingle();
                        XUV2Offset = Stream.ReadSingle();
                        XUV2Scale = Stream.ReadSingle();
                        YUV2Offset = Stream.ReadSingle();
                        YUV2Scale = Stream.ReadSingle();
                        Stream.Position += 36;
                        break;
                    }
                default: throw new NotSupportedException();
            }

            Name = Stream.ReadASCII(Stream.ReadInt32());
            Lod = Stream.ReadInt32();
            IndexType = (IndexType)Stream.ReadUInt32();
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 1);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 0);
            Utilities.AssertEquals(Stream.ReadSingle(), 0);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadUInt32(), 3);
            Indices = Utilities.ReadIndices(Stream, Stream.ReadInt32(), Stream.ReadInt32());
            Stream.Position += 4;
            FaceCount = Utilities.CalculateFaceCount(Indices, IndexType);
            VertexCount = Utilities.CalculateVertexCount(Indices);
        }

        #endregion
    }
}
