﻿/*
    Forza Studio: Forza Motorsport resource extraction tool.

    Copyright (C) 2012 Mike Davis (http://codeescape.com/)

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be included 
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace ForzaStudio
{
    public class ForzaMesh : IDisposable
    {
        #region Fields
        public string Name;
        public string MaterialName;
        public int[] Indices;
        public ForzaVertex[] Vertices;
        #endregion

        #region Properties
        public int FaceCount
        {
            get
            {
                return Indices.Length / 3;
            }
        }
        public int VertexCount
        {
            get
            {
                return Vertices.Length;
            }
        }
        #endregion

        #region Constructor / Destructor
        public ForzaMesh(string name, string materialName, int[] indices, ForzaVertex[] vertices)
        {
            Name = name;
            MaterialName = materialName;
            Indices = indices;
            Vertices = vertices;
        }

        ~ForzaMesh()
        {
            Dispose();
        }
        #endregion

        #region Methods
        public void Draw(ref GraphicsDevice device)
        {
            // bottleneck is here: good thing this tool is for extraction and not rendering :X
            // TODO: batch up all primitives and draw at once perhaps? also trim up the ForzaVertex declaration
            device.VertexDeclaration = new VertexDeclaration(device, ForzaVertex.VertexElements);
            device.DrawUserIndexedPrimitives<ForzaVertex>(PrimitiveType.TriangleList, Vertices, 0, Vertices.Length, Indices, 0, FaceCount);
        }

        public void Dispose()
        {
            Name = null;
            MaterialName = null;
            Indices = null;
            Vertices = null;
        }
        #endregion
    }
}
    

