﻿/*
    Forza Studio: Forza Motorsport resource extraction tool.

    Copyright (C) 2012 Mike Davis (http://codeescape.com/)

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be included 
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ForzaStudio
{
    public class ForzaTrackBin : ForzaResource
    {
        #region Fields
        public EndianStream Stream;
        public ForzaTrackSection[] Sections;
        public string Type;
        #endregion

        #region Constructors / Destructor
        public ForzaTrackBin(string archivePath, string filePath)
            : base(filePath, archivePath)
        {
            Initialize();
        }

        public ForzaTrackBin(string filePath)
            : base(filePath)
        {
            Initialize();
        }

        ~ForzaTrackBin()
        {
            Dispose();
        }
        #endregion

        #region Methods

        // working for forza 4 tracks
        public void Initialize()
        {
            Stream = new EndianStream(GetDataStream(), EndianType.BigEndian);
            Name = FileName;
            uint type = Stream.ReadUInt32();
            if (type == 4) Version = ForzaVersion.Three;
            else if (type == 6) Version = ForzaVersion.Four;
            Stream.Position += 112;
            uint subSectionCount = Stream.ReadUInt32();
            Sections = new ForzaTrackSection[subSectionCount];
            for (int i = 0; i < Sections.Length; i++)
            {
                Sections[i] = new ForzaTrackSection(this);
            }
            Stream.Dispose();
        }

        public override void Dispose()
        {
            if (Stream != null) Stream.Dispose();
            Sections = null;
        }

        #endregion
    }
}
