﻿/*
    Forza Studio: Forza Motorsport resource extraction tool.

    Copyright (C) 2012 Mike Davis (http://codeescape.com/)

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be included 
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ForzaStudio
{
    // 82 road sections in amalfi f3

    // subsection contains bounding box information used to scale down vertices
    // then apply the offset

    public class ForzaTrackSection
    {
        public ForzaTrackBin Parent;
        public EndianStream Stream;
        public ForzaTrackSubSection[] SubSections;
        public string Name;
        public string Type;
        public Vector3 Offset;  // position of the center of the bounding box

        public ForzaTrackSection(ForzaTrackBin parent)
        {
            Parent = parent;
            Stream = parent.Stream;
            Utilities.AssertEquals(Stream.ReadUInt32(), 1);
            Vector3 Offset = new Vector3(Stream.ReadSingle(), Stream.ReadSingle(), Stream.ReadSingle());
            Utilities.AssertEquals(Stream.ReadSingle(), 0);
            Vector3 targetMin = new Vector3(Stream.ReadSingle(), Stream.ReadSingle(), Stream.ReadSingle());
            Utilities.AssertEquals(Stream.ReadSingle(), 0);
            Vector3 targetMax = new Vector3(Stream.ReadSingle(), Stream.ReadSingle(), Stream.ReadSingle());
            Utilities.AssertEquals(Stream.ReadSingle(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 1);
            Name = Stream.ReadASCII(Stream.ReadInt32()).ToLowerInvariant();    // the full name
            string[] nameParts = Name.Split(new char[] { '_' });
            Type = nameParts[0].ToLowerInvariant();
            Name = Name.Substring(Name.IndexOf("_") + 1);
            Utilities.AssertEquals(Stream.ReadUInt32(), 2);
            uint vertexCount = Stream.ReadUInt32();
            uint vertexSize = Stream.ReadUInt32();
            ForzaVertex[] vertices = new ForzaVertex[vertexCount];
            for (int i = 0; i < vertexCount; i++)
            {
                vertices[i] = new ForzaVertex(parent.Version, ForzaVertexType.Track, Stream, vertexSize);
            }
            Utilities.AssertEquals(Stream.ReadUInt32(), 1);

            BoundingBox bounds = BoundingBox.FromVertices(vertices);

            SubSections = new ForzaTrackSubSection[Stream.ReadUInt32()];
            for (int i = 0; i < SubSections.Length; i++)
            {
                ForzaTrackSubSection subSection = new ForzaTrackSubSection(this);
                subSection.Vertices = Utilities.GenerateVertices(vertices, ref subSection.Indices);
                for (int j = 0; j < subSection.Vertices.Length; j++)
                {
                    //subSection.Vertices[j].position = Utilities.CalculateBoundTargetValue(subSection.Vertices[j].position, bounds.Min, bounds.Max, targetMin, targetMax);
                    //subSection.Vertices[j].position += Offset;

                    subSection.Vertices[j].texture0.Y = 1.0f - subSection.Vertices[j].texture0.Y;
                    subSection.Vertices[j].texture1.Y = 1.0f - subSection.Vertices[j].texture1.Y;
                }
                if (subSection.IndexType == IndexType.TriStrip)
                {
                    subSection.Indices = Utilities.GenerateTriangleList(subSection.Indices, subSection.FaceCount);
                }
                SubSections[i] = subSection;
            }
        }
    }
}
