﻿/*
    Forza Studio: Forza Motorsport resource extraction tool.

    Copyright (C) 2012 Mike Davis (http://codeescape.com/)

    Permission is hereby granted, free of charge, to any person 
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be included 
    in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ForzaStudio
{
    public class ForzaTrackSubSection
    {
        public ForzaTrackSection Parent;
        public EndianStream Stream;
        public string Name;
        public uint Lod;
        public IndexType IndexType;
        public int[] Indices;
        public ForzaVertex[] Vertices;
        public int VertexCount;
        public int FaceCount;
        public Vector3 Offset;

        public ForzaTrackSubSection(ForzaTrackSection parent)
        {
            Parent = parent;
            Stream = parent.Stream;
            Utilities.AssertEquals(Stream.ReadUInt32(), 1);
            Utilities.AssertEquals(Stream.ReadUInt32(), 2);
            Name = Stream.ReadASCII(Stream.ReadInt32()).ToLowerInvariant();
            Lod = Stream.ReadUInt32();
            IndexType = (IndexType)Stream.ReadUInt32();
            if (IndexType != IndexType.TriList && IndexType != IndexType.TriStrip)
            {
                throw new Exception("analyze this!");
            }
            uint unkType = Stream.ReadUInt32();  // 0, 1, 2
            Utilities.AssertEquals(Stream.ReadUInt32(), 1);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadUInt32(), 0);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            Utilities.AssertEquals(Stream.ReadSingle(), 1);
            float f5 = Stream.ReadSingle();
            float f6 = Stream.ReadSingle();
            float f7 = Stream.ReadSingle();
            float f8 = Stream.ReadSingle();

            Utilities.AssertEquals(Stream.ReadUInt32(), 3);
            Indices = Utilities.ReadIndices(Stream, Stream.ReadInt32(), Stream.ReadInt32());
            unkType = Stream.ReadUInt32();
            if (unkType != 0 && unkType != 1 && unkType != 2 && unkType != 5)
            {
                throw new Exception("analyze this!");
            }
            VertexCount = Utilities.CalculateVertexCount(Indices);
            FaceCount = Utilities.CalculateFaceCount(Indices, IndexType);
        }
    }
}
