#include <windows.h>
#include <stdio.h>
#include "d3d9.h"      // Note: make sure this is the Xbox 360 flavor of D3D9.h!
#include "xgraphics.h"
#include "d3dx9math.h"

DWORD GetMaskFromFormat( D3DFORMAT Format )
{
    switch( Format|D3DFORMAT_TILED_MASK )
    {
        case D3DFMT_DXT1:                 return 0xffffffff;
        case D3DFMT_DXT3:                 return 0xffffffff;
        case D3DFMT_DXT5:                 return 0xffffffff;

        case D3DFMT_A8:                   return 0xff000000;
        case D3DFMT_L8:                   return 0x00ffffff;

        case D3DFMT_R5G6B5:               return 0x00ffffff;
        case D3DFMT_R6G5B5:               return 0x00ffffff;
        case D3DFMT_L6V5U5:               return 0x00ffffff;

        case D3DFMT_X1R5G5B5:             return 0x00ffffff;
        case D3DFMT_A1R5G5B5:             return 0xffffffff;
        case D3DFMT_A4R4G4B4:             return 0x00ffffff;
        case D3DFMT_X4R4G4B4:             return 0xffffffff;

        case D3DFMT_A8L8:                 return 0xffffffff;
        case D3DFMT_V8U8:                 return 0x00ffff00;
//        case D3DFMT_CxV8U8:               return 0x00ffff00;

        case D3DFMT_D16:                  return 0x00ffffff;
        case D3DFMT_L16:                  return 0x00ffffff;

        case D3DFMT_R16F:                 return 0x00ff0000;

        case D3DFMT_UYVY:                 return 0x00ffffff;
        case D3DFMT_G8R8_G8B8:            return 0x00ffffff;
        case D3DFMT_R8G8_B8G8:            return 0x00ffffff;
        case D3DFMT_YUY2:                 return 0x00ffffff;

        case D3DFMT_A8R8G8B8:             return 0xffffffff;
        case D3DFMT_X8R8G8B8:             return 0x00ffffff;
        case D3DFMT_A8B8G8R8:             return 0xffffffff;
        case D3DFMT_X8B8G8R8:             return 0x00ffffff;
        case D3DFMT_X8L8V8U8:             return 0x00ffffff;
        case D3DFMT_Q8W8V8U8:             return 0xffffffff;

        case D3DFMT_A2R10G10B10:          return 0xffffffff;
        case D3DFMT_A2B10G10R10:          return 0xffffffff;
        case D3DFMT_A2W10V10U10:          return 0xffffffff;

        case D3DFMT_G16R16:               return 0x00ffff00;
        case D3DFMT_V16U16:               return 0x00ffff00;

        case D3DFMT_R10G11B11:            return 0x00ffffff;

        case D3DFMT_R11G11B10:            return 0x00ffffff;
        case D3DFMT_W11V11U10:            return 0x00ffffff;

        case D3DFMT_G16R16F:              return 0x00ffff00;

        case D3DFMT_R32F:                 return 0x00ff0000;

        case D3DFMT_A16B16G16R16:         return 0xffffffff;
        case D3DFMT_Q16W16V16U16:         return 0xffffffff;
        case D3DFMT_A16B16G16R16F:        return 0xffffffff;
        case D3DFMT_G32R32F:              return 0x00ffff00;

        case D3DFMT_A32B32G32R32F:        return 0xffffffff;

        case D3DFMT_A2B10G10R10F_EDRAM:   return 0xffffffff;

        case D3DFMT_DXN:                  return 0x00ffff00;
        case D3DFMT_CTX1:                 return 0x00ffff00;

        case D3DFMT_D24S8:                return 0xffffffff;
        case D3DFMT_D24X8:                return 0x00ffffff;
        case D3DFMT_D24FS8:               return 0xffffffff;
        case D3DFMT_D32:                  return 0x00ffffff;
    }

    return 0xffffffff;
}

//--------------------------------------------------------------------------------------
// Name: Create
// Desc: Loads first texture resource from the given XDS file data.
//--------------------------------------------------------------------------------------
extern "C" __declspec( dllexport ) void Create(BYTE* inputData, int inputSize, BYTE* outputData)
{
    // Compute memory requirements
    DWORD m_dwSysMemDataSize = 100;
    DWORD m_dwVidMemDataSize = inputSize - 52;

    // Allocate memory
    BYTE* m_pSysMemData = inputData;
    BYTE* m_pVidMemData = inputData + 4096;

    // Fixup the texture memory
    D3DTexture* pTexture = (D3DTexture*)&m_pSysMemData[48];
    XGEndianSwapMemory(pTexture, pTexture, XGENDIAN_8IN32, sizeof(DWORD), sizeof(D3DTexture) / sizeof(DWORD));

    // Adjust Base address according to where memory was allocated
    XGOffsetBaseTextureAddress(pTexture, m_pVidMemData, m_pVidMemData);

	// Get information about the texture
    XGTEXTURE_DESC desc;
    XGGetTextureDesc(pTexture, 0, &desc);

	// Get the base size and mip size
	UINT  dwBaseSize;
	UINT  dwMipSize;
	DWORD Usage = 0;
	DWORD Flags = 0;
	UINT Pitch = pTexture->Format.Pitch;
	if(!pTexture->Format.PackedMips) Flags |= XGHEADEREX_NONPACKED;
	if(pTexture->Format.BorderSize) Flags |= XGHEADEREX_BORDER;
	DWORD dwBaseOffset = pTexture->Format.BaseAddress << 12;
	DWORD dwMipOffset = pTexture->Format.MipAddress  << 12;

	D3DTexture texHeader;
	XGSetTextureHeaderEx( desc.Width, desc.Height, 1, Usage, 
						  desc.Format, desc.ExpBias, Flags,
						  dwBaseOffset, dwMipOffset, Pitch,
						  &texHeader, &dwBaseSize, &dwMipSize );

	// grab the first level (full quality) texture
	BYTE* pSrcBits;
	DWORD dwSlicePitch = desc.SlicePitch;	// texture size
	DWORD dwRowPitch = desc.RowPitch;
	DWORD dwWidth = desc.Width;
	DWORD dwHeight = desc.Height;
	DWORD dwWidthInBlocks = desc.WidthInBlocks;
	DWORD dwHeightInBlocks = desc.HeightInBlocks;
	DWORD dwBytesPerBlock = desc.BytesPerBlock;

	// Calculate the mip level offset
	pSrcBits = (BYTE*)((pTexture->Format.BaseAddress << 12));

	// Untile the source texels, if necessary
	BYTE* m_pUntiledBits = new BYTE[ dwSlicePitch ];
	if( desc.Format & D3DFORMAT_TILED_MASK )
	{
		POINT point = { 0 };
		RECT rect = { 0, 0, dwWidthInBlocks, dwHeightInBlocks };
		XGUntileSurface( m_pUntiledBits, dwRowPitch, &point, pSrcBits, dwWidthInBlocks, dwHeightInBlocks, &rect, dwBytesPerBlock );
	}
	else memcpy( m_pUntiledBits, pSrcBits, dwSlicePitch );

	// Do the format conversion
	D3DFORMAT SrcFormat = (D3DFORMAT)( desc.Format & ~D3DFORMAT_TILED_MASK);
	D3DFORMAT DstFormat = D3DFMT_LIN_A8R8G8B8;
	DstFormat = (D3DFORMAT)(((DWORD)DstFormat & ~D3DFORMAT_ENDIAN_MASK) | (GPUENDIAN_NONE << D3DFORMAT_ENDIAN_SHIFT));
	XGCopySurface( (DWORD*)outputData, sizeof(DWORD)*dwWidth, dwWidth, dwHeight,
				   DstFormat, NULL, m_pUntiledBits, dwRowPitch, SrcFormat, NULL, 0, 0.0f );

	delete[] m_pUntiledBits;

	// For convenience when viewing the resulting unbundled textures, let's
	// clear out any unused channels
	DWORD dwMask = GetMaskFromFormat( desc.Format );
	if(dwMask != 0xffffffff)
	{
		DWORD* pData = (DWORD*)outputData;
		DWORD dwAlphaSet = 0xff000000;
		if(dwMask & 0xff000000) dwAlphaSet = 0;
		for( DWORD y=0; y<dwHeight; y++ )
			for( DWORD x=0; x<dwWidth; x++ )
				*pData++ = ( (*pData) & dwMask ) | dwAlphaSet;
	}
}

extern "C" __declspec( dllexport ) void GetTextureSize(BYTE* inputData, int inputSize, int* outputWidth, int* outputHeight)
{
    // Fixup the texture memory
    D3DTexture* pTexture = (D3DTexture*)&inputData[48];
    XGEndianSwapMemory( pTexture, pTexture, XGENDIAN_8IN32, sizeof(DWORD), sizeof(D3DTexture)/sizeof(DWORD) );

	// Get information about the first texture
	XGTEXTURE_DESC desc;
	XGGetTextureDesc(pTexture, 0, &desc);

	XGEndianSwapMemory( pTexture, pTexture, XGENDIAN_8IN32, sizeof(DWORD), sizeof(D3DTexture)/sizeof(DWORD) );

	// save buffer size
	*outputWidth = desc.Width;
	*outputHeight = desc.Height;
}

// pitch is the number of bytes a line of pixels takes up
extern "C" __declspec( dllexport ) int ConvertToBitmap(BYTE* inputData, int inputSize, D3DFORMAT d3dFormat, int width, int height, BYTE* outputData)
{
	int widthInBlocks, heightInBlocks, bytesPerBlock, rowPitch;
	widthInBlocks = width / 4;
	heightInBlocks = height / 4;

	// calculate image information
	if (d3dFormat == D3DFMT_DXT1 || d3dFormat == D3DFMT_DXT5A || d3dFormat == D3DFMT_DXT3A_1111 ||
		d3dFormat == D3DFMT_LIN_DXT5A + 0x7F00 || d3dFormat == D3DFMT_LIN_DXT1 + 0x7F00)
	{
		bytesPerBlock = 8;
		rowPitch = width * 2;
	}
	else if (d3dFormat == D3DFMT_DXT4 || d3dFormat == D3DFMT_DXN || d3dFormat == D3DFMT_LIN_DXT4 + 0x7F00 || 
		d3dFormat == D3DFMT_LIN_DXT2 + 0x7F00 || d3dFormat == D3DFMT_LIN_DXN + 0x7F00)
	{
		bytesPerBlock = 16;
		rowPitch = width * 4;
	}
	else if (d3dFormat == D3DFMT_L8 || d3dFormat == D3DFMT_LIN_L8 + 0x7F00)
	{
		bytesPerBlock = 1;
		rowPitch = width;
		widthInBlocks = width;
		heightInBlocks = height;
	}
	else if (d3dFormat == D3DFMT_LIN_R5G6B5 + 0x7F00 || d3dFormat == D3DFMT_A8L8 || d3dFormat == D3DFMT_LIN_A8L8 + 0x7F00)
	{
		bytesPerBlock = 2;
		rowPitch = width * 2;
		widthInBlocks = width;
		heightInBlocks = height;
	}
	else if (d3dFormat == D3DFMT_X8R8G8B8)
	{
		// hack hack hack
		memcpy(outputData, inputData, inputSize);
		goto DoneConverting;
	}
	else return 0;

	// Untile the source texels, if necessary
	BYTE* m_pUntiledBits = new BYTE[inputSize];
	if( d3dFormat & D3DFORMAT_TILED_MASK )
	{
		POINT point = { 0 };
		RECT rect = { 0, 0, widthInBlocks, heightInBlocks };
		XGUntileSurface( m_pUntiledBits, rowPitch, &point, inputData, widthInBlocks, heightInBlocks, &rect, bytesPerBlock );
	}
	else memcpy( m_pUntiledBits, inputData, inputSize );

	// Do the format conversion
	D3DFORMAT SrcFormat = (D3DFORMAT)( d3dFormat & ~D3DFORMAT_TILED_MASK);
	D3DFORMAT DstFormat = D3DFMT_LIN_A8R8G8B8;
	DstFormat = (D3DFORMAT)(((DWORD)DstFormat & ~D3DFORMAT_ENDIAN_MASK) | (GPUENDIAN_NONE << D3DFORMAT_ENDIAN_SHIFT));
	XGCopySurface( (DWORD*)outputData, sizeof(DWORD)*width, width, height,
				   DstFormat, NULL, m_pUntiledBits, rowPitch, SrcFormat, NULL, 0, 0.0f );

	delete[] m_pUntiledBits;

DoneConverting:

	// For convenience when viewing the resulting unbundled textures, let's
	// clear out any unused channels
	DWORD dwMask = GetMaskFromFormat( d3dFormat );
	if(dwMask != 0xffffffff)
	{
		DWORD* pData = (DWORD*)outputData;
		DWORD dwAlphaSet = 0xff000000;
		if(dwMask & 0xff000000) dwAlphaSet = 0;
		for( DWORD y=0; y<height; y++ )
			for( DWORD x=0; x<width; x++ )
				*pData++ = ( (*pData) & dwMask ) | dwAlphaSet;
	}
	return 1;
}